﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using KreslikLib.Common;

namespace ConvertFPIRings
{
    public partial class ConvertWindow : Form
    {
        string inputFile;
        string outputFile;
        bool inputFileOK;
        bool outputFileOK;

        public ConvertWindow()
        {
            inputFileOK = false;
            outputFileOK = false;

            InitializeComponent();
        }

        private void button1_Click(object sender, EventArgs e)
        {
            if (inputFileOK && outputFileOK)
            {
                this.statusBar.Text = RunConversion(inputFile, outputFile) + " rings processed";
            }
            else
            {
                KTools.InfoBox("Please select the input and output files first.", "Select files first");
            }
        }

        private string RunConversion(string inputFile, string outputFile)
        {
            string[] ExcelColumn = { "B", "C", "D", "E", "F", "G", "H", "I", "J", "K", "L", "M", "N", "O" };
            string inputLine;
            string outputLineSymbols;
            string outputLineFormulas;
            string currentPortion;
            string FPIFormula;
            int symbolCount;
            int lineNumber = 1;
            Exception e = null;

            StreamReader Reader = null;
            StreamWriter Writer = null;

            try
            {
                Reader = new StreamReader(inputFile);
                Writer = new StreamWriter(outputFile, false);

                while (!Reader.EndOfStream)
                {
                    inputLine = Reader.ReadLine();
                    outputLineSymbols = "FPI:,";
                    outputLineFormulas = ",";
                    symbolCount = SymbolCount(inputLine);
                    FPIFormula = "=";

                    for (int i = 0; i < symbolCount; i++)
                    {
                        currentPortion = KTools.PortionFromCSVLine(inputLine, i, StartFrom.Begin);

                        outputLineSymbols += ParenthesisToBidAsk(currentPortion);
                        outputLineFormulas += "1";

                        if (PairIsBuy(currentPortion))
                        {
                            FPIFormula += ExcelColumn[i] + (lineNumber * 3 - 1);
                        }
                        else
                        {
                            FPIFormula += "(1/" + ExcelColumn[i] + (lineNumber * 3 - 1) + ")";
                        }

                        if (i < symbolCount - 1)
                        {
                            outputLineSymbols += ",";
                            outputLineFormulas += ",";
                            FPIFormula += "*";
                        }
                    }

                    Writer.WriteLine(outputLineSymbols);
                    Writer.WriteLine(FPIFormula + outputLineFormulas + "\n");

                    lineNumber++;
                }

                KTools.InfoBox("FPI rings conversion done.\n" + (lineNumber - 1).ToString() + " rings processed.", "Conversion done");
            }
            catch (Exception E)
            {
                KTools.InfoBox(E.Message, "Error");
                e = E;
            }
            finally
            {
                if (Reader != null)
                {
                    Reader.Dispose();
                }

                if (Writer != null)
                {
                    Writer.Dispose();
                }
            }

            return (lineNumber - 1).ToString();
        }

        private string ParenthesisToBidAsk(string SymbolWithPar)
        {
            if (PairIsBuy(SymbolWithPar))
            {
                return SymbolWithPar + " (Ask):";
            }
            else
                return SymbolWithPar.Substring(1, 7) + " (Bid):";
        }

        private bool PairIsBuy(string SymbolWithPar)
        {
            if (SymbolWithPar[0] == '(')
                return false;
            else
                return true;
        }

        private int SymbolCount(string Line)
        {
            int charCount = 0;
            char lineSeparationChar = ',';

            for (int lineChar = 0; lineChar < Line.Length; lineChar++)
            {
                if (Line[lineChar] == lineSeparationChar)
                    charCount++;
            }

            return charCount + 1;
        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            this.lblInput.Text = inputFile = openFileDialog1.FileName;
            inputFileOK = true;
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            this.lblOutput.Text = outputFile = saveFileDialog1.FileName;
            outputFileOK = true;
        }

        private void openToolStripButton_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
        }

        private void saveToolStripButton_Click(object sender, EventArgs e)
        {
            saveFileDialog1.ShowDialog();
        }
    }
}